
/* polysyndeton
 * 1999 (c) scut / teso
 *
 * main routines
 */

#define	B4A_MAIN_C

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "relay.h"
#include "client.h"
#include "main.h"
#include "network.h"

void	srv_chooseproxy (char **host, unsigned short int *port);
int	srv_readnlines (FILE *f, int p);
int	srv_proxycount (FILE *f);

bound	*b;
char	*pfile = "proxies";

int
main (int argc, char **argv)
{
	int	n;

	banner ();

	if (argc != 3) {
		usage ();
		exit (EXIT_FAILURE);
	}
/*	if (fork () != 0)
		exit (0);
*/

	b = net_bind (argv[1], atoi(argv[2]));
	if (b == NULL) {
		printf ("cannot bind to %s:%u\n", argv[1], atoi(argv[2]));
		quit();
	}
	printf("bound to %s, port %u\n", argv[1], atoi(argv[2]));

	srv_main ();

	/* should never happen */
	return (0);
}

void
srv_main (void)
{
	struct sockaddr_in	csa;
	client			*cnew;
	int			cs, n;

	while (1) {
		cs = net_accept (b->bs, &csa, 20);
		if (cs == -1) {
			quit();
		}

		/* if we actually experience a new connection start a new client thread */
		if (cs) {
			cnew = cl_add ();
			if (cnew == NULL) {
				printf ("cannot add new client\n");
			} else {
				pthread_mutex_lock (&cnew->cl_mutex);
				cnew->cs = cs;

				srv_chooseproxy (&cnew->connip, &cnew->connport);

				memcpy (&cnew->csa, &csa, sizeof (struct sockaddr_in));
				pthread_mutex_unlock (&cnew->cl_mutex);

				n = pthread_create (&cnew->tid, NULL, (void *) cl_handle, (void *) cnew);
				if (n == -1) {
					printf ("cannot create new client thread: %s\n", strerror (errno));
				}
			}
		}
	}

	quit();
}


void
srv_chooseproxy (char **host, unsigned short int *port)
{
	FILE	*proxylist;
	int	n, m;
	char	buff[1024];

	proxylist = fopen (pfile, "r");
	if (proxylist == NULL) {
		printf ("cannot find proxy list %s\n", pfile);
		exit (EXIT_FAILURE);
	}

	n = srv_proxycount (proxylist);
	m = (random () % n) - 1;
	srv_readnlines (proxylist, m);
	fgets (buff, sizeof (buff) - 1, proxylist);

	if (net_parseip (buff, host, port) == 0) {
		printf ("invalid proxy entry: %s\n", buff);
		exit (EXIT_FAILURE);
	}

	return;
}


int
srv_readnlines (FILE *f, int p)
{
	char	buff[1024];

	while (fgets (buff, sizeof (buff), f) != NULL && (p-- > 0))
		;

	return;
}


int
srv_proxycount (FILE *f)
{
	char	buff[1024];
	int	n = 0;

	while (fgets (buff, sizeof (buff), f) != NULL)
		n++;

	fseek (f, 0, SEEK_SET);

	return (n);
}


void
usage (void)
{
	printf("usage: polysyndeton <local-ip | *> <local-port>\n\n");
	return;
}

void
banner (void)
{
	printf("polysyndeton "VERSION" by "AUTHORS"\n\n");
	return;
}

void
quit (void)
{
	printf ("shutting polysyndeton down.\n");

	/* terminate main thread */
	exit (0);
}

